@@ -1,5 +1,6 @@
 import os
 import subprocess
+import time
 import logging
 
 import requests
@@ -32,7 +33,9 @@ from .utils import (
     UseTlsAdapter,
     get_sub_vs_list_from_data,
     get_api_bool_string,
-    get_dict_entry_as_list)
+    get_dict_entry_as_list,
+    cast_to_list,
+    falsey_to_none)
 
 
 requests.packages.urllib3.disable_warnings()
@@ -238,13 +241,12 @@ class LoadMaster(HttpClient):
         response = self._get("/listvs")
         data = get_data(response)
         virtual_services = []
-        services = data.get('VS', [])
         # if there is no VS key, build_virtual_services will fail with a
         # ValidationError, which is the best we can do for now
         # (without changing the upstream code and raising an exception earlier,
         # possibly retrying)
-        if not isinstance(services, list):
-            services = [services]
+        services = data.get('VS', [])
+        services = cast_to_list(services)
         for service in services:
             master_vs_id = int(service.get('MasterVSID', 0))
             if master_vs_id != 0:
@@ -278,13 +280,11 @@ class LoadMaster(HttpClient):
         response = self._get("/listvs")
         data = get_data(response)
         virtual_services_data = data.get('VS', [])
-        if not isinstance(virtual_services_data, list):
-            virtual_services_data = [virtual_services_data]
+        virtual_services_data = cast_to_list(virtual_services_data)
         # loop through all vss and gather real server objects
         for service_data in virtual_services_data:
             real_servers_data = service_data.get('Rs', [])
-            if not isinstance(real_servers_data, list):
-                real_servers_data = [real_servers_data]
+            real_servers_data = cast_to_list(real_servers_data)
             global_real_servers_data.extend(real_servers_data)
 
         # create vs and rs objects at this point
@@ -294,7 +294,8 @@ class LoadMaster(HttpClient):
             if master_vs_id != 0:
                 for vs in virtual_services_data:
                     if int(vs.get("Index", 0)) == master_vs_id:
-                        virt_serv = self.build_virtual_service(service_data, vs)
+                        virt_serv = self.build_virtual_service(service_data,
+                                                               vs)
             else:
                 virt_serv = self.build_virtual_service(service_data, response)
             virtual_services.append(virt_serv)
@@ -328,9 +329,11 @@ class LoadMaster(HttpClient):
         """
         is_sub_vs = True if int(service.get('MasterVSID', 0)) != 0 else False
         if is_sub_vs:
+            # `response` needs to be a dict in here
             # Add lb properties to the sub vs
             if response is None:
-                response = self._get("/showvs", {"vs": service.get('MasterVSID')})
+                response = self._get("/showvs",
+                                     {"vs": service.get('MasterVSID')})
                 parent_vs_data = get_data(response)
             else:
                 parent_vs_data = response
@@ -340,6 +343,7 @@ class LoadMaster(HttpClient):
             virt_serv.subvs_data = subvs_lb_props[service.get('Index')]
             virt_serv.subvs_data['parentvs'] = service.get('MasterVSID')
         else:
+            # `response` needs to be a raw xml output here
             # Add any sub VSs to the top level VS
             if response is None:
                 response = self._get("/listvs")
@@ -357,14 +361,13 @@ class LoadMaster(HttpClient):
                 if vs['MasterVSID'] == this_vs_index:
                     subvs = VirtualService(self.access_info, vs['Index'])
                     subvs.populate_default_attributes(vs)
-                    #subvs_api_entries = get_dict_entry_as_list("SubVS", data)
                     subvs_api_entries = service.get("SubVS", [])
-                    if not isinstance(subvs_api_entries, list):
-                        subvs_api_entries = [subvs_api_entries]
+                    subvs_api_entries = cast_to_list(subvs_api_entries)
                     for subvs_api in subvs_api_entries:
                         # add the "Rs" part of the subvs to the subvs instance
                         if subvs_api["VSIndex"] == subvs.index:
                             subvs.subvs_data = subvs_api
+                            # Have to add a parentvs hook to make life easy
                             subvs.subvs_data['parentvs'] = this_vs_index
                     virt_serv.subvs_entries.append(subvs)
         virt_serv.populate_default_attributes(service)
@@ -520,7 +523,8 @@ class LoadMaster(HttpClient):
     def backup(self, location='backups'):
         if not os.path.exists(location):
             os.makedirs(location)
-        file_name = os.path.join(location, "{}.backup".format(self.ip_address))
+        file_name = os.path.join(location, "{}_{}.backup".format(
+            self.ip_address, time.strftime("%Y-%m-%d_%H:%M:%S")))
 
         with open(file_name, 'wb') as file:
             cmd = self._get_curl_command_list('backup')
@@ -586,8 +590,7 @@ class LoadMaster(HttpClient):
 
     def set_user_perms(self, user, perms=None):
         perms = [] if perms is None else perms
-        if not isinstance(perms, list):
-            perms = [perms]
+        perms = cast_to_list(perms)
         params = {
             'user': user,
             'perms': ",".join([perm for perm in perms]),
@@ -809,6 +812,24 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         self.qos = service.get('QoS', None)
         self.idletime = service.get('Idletime', None)
 
+        # Real servers section
+        self.checktype = service.get('CheckType', None)
+        self.checkhost = service.get('CheckHost', None)
+        self.checkpattern = service.get('CheckPattern', None)
+        self.checkurl = service.get('CheckUrl', None)
+        self.checkcodes = service.get('CheckCodes', None)
+        self.checkheaders = service.get('CheckHeaders', None)
+        self.matchlen = service.get('MatchLen', None)
+        self.__dict__['checkuse1.1'] = service.get('CheckUse1.1', None)
+        self.checkport = falsey_to_none(int(service.get('CheckPort', 0)))
+        self.checkuseget = service.get('CheckUseGet', None)
+        self.extrahdrkey = service.get('ExtraHdrKey', None)
+        self.extrahdrvalue = service.get('ExtraHdrValue', None)
+        self.checkpostdata = service.get('CheckPostData', None)
+        self.rsruleprecedence = service.get('RSRulePrecedence', None)
+        self.rsruleprecedencepos = service.get('RSRulePrecedencePos', None)
+        self.enhancedhealthchecks = service.get('EnhancedHealthchecks', None)
+
     def create_sub_virtual_service(self):
         """VirtualService factory with pre-configured LoadMaster connection
 
@@ -858,8 +879,7 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         data = get_data(response)
         real_servers = []
         servers = data.get('Rs', [])
-        if not isinstance(servers, list):
-            servers = [servers]
+        servers = cast_to_list(servers)
         for server in servers:
             real_server = self.build_real_server(server)
             real_servers.append(real_server)
